import java.util.*;
import java.awt.geom.*;

public class ConvexHull {

    private final List<Point2D> _points;

    public ConvexHull() {
        _points = new ArrayList<Point2D>();
    }

    public List<Point2D> getHull() {
        final int n = _points.size();
        if (n == 0) {
            return Collections.emptyList();
        } else if (n == 1) {
            return Collections.singletonList(_points.get(0));
        } else {
            Collections.sort(_points, new OrderOnX());

            List<Point2D> upper = new ArrayList<>();
            upper.add(_points.get(0));
            upper.add(_points.get(1));
            for(int i = 2; i < n; i++) {
                upper.add(_points.get(i));
                removeRightTurns(upper);
            }

            List<Point2D> lower = new ArrayList<>();
            lower.add(_points.get(n - 1));
            lower.add(_points.get(n - 2));
            for(int i = n - 3; i >= 0; i--) {
                lower.add(_points.get(i));
                removeRightTurns(lower);
            }

            lower.remove(0);
            lower.remove(lower.size() - 1);
            upper.addAll(lower);
            return upper;
        }
    }

    private void removeRightTurns(final List<Point2D> list) {
        while (list.size() > 2) {
            int n = list.size();
            Point2D to = list.get(n - 1);
            Point2D via = list.get(n - 2);
            Point2D from = list.get(n - 3);
            if(makesRightTurnOrColinear(from, via, to)) {
                list.remove(via);
            } else {
                return;
            }
        }
    }

    private boolean makesRightTurnOrColinear(final Point2D from, final Point2D via, final Point2D to) {
        Line2D line = new Line2D.Double(from, via);
        return line.relativeCCW(to) <= 0;
    }

    public void add(final Point2D p) {
        _points.add(p);
    }

    private class OrderOnX implements Comparator<Point2D> {
        @Override
        public int compare(final Point2D here, final Point2D there) {
            return Double.compare(here.getX(), there.getX());
        }
    }

}
