import java.util.*;
import javax.vecmath.*;
import javax.swing.*;

public class Cube {

    private LineSegment _l1;
    private LineSegment _l2;
    private LineSegment _l3;
    private LineSegment _l4;
    private LineSegment _l5;
    private LineSegment _l6;
    private LineSegment _l7;

    public Cube() {
        _l1 = new LineSegment(new Point3d(-0.5, -0.5, -0.5),
                              new Point3d(0.5, -0.5, -0.5));
        _l2 = new LineSegment(new Point3d(-0.5, 0.5, -0.5),
                              new Point3d(0.5, 0.5, -0.5));
        _l3 = new LineSegment(new Point3d(-0.5, -0.5, -0.5),
                              new Point3d(-0.5, 0.5, -0.5));
        _l4 = new LineSegment(new Point3d(0.5, -0.5, -0.5),
                              new Point3d(0.5, 0.5, -0.5));
        _l5 = new LineSegment(new Point3d(-0.5, -0.5, 0.5),
                              new Point3d(0.5, -0.5, 0.5));
        _l6 = new LineSegment(new Point3d(-0.5, 0.5, 0.5),
                             new Point3d(0.5, 0.5, 0.5));
        _l7 = new LineSegment(new Point3d(-0.5, -0.5, 0.5),
                              new Point3d(-0.5, 0.5, 0.5));
    }

    public void rotate(final double alpha, final double beta, final double gamma) {
        _l1.rotate(alpha, beta, gamma);
        _l2.rotate(alpha, beta, gamma);
        _l3.rotate(alpha, beta, gamma);
        _l4.rotate(alpha, beta, gamma);
        _l5.rotate(alpha, beta, gamma);
        _l6.rotate(alpha, beta, gamma);
        _l7.rotate(alpha, beta, gamma);
    }

    public void intersectXYPlane(final double h, final ConvexHull hull) {
        _l1.intersectXYPlane(h, hull);
        _l2.intersectXYPlane(h, hull);
        _l3.intersectXYPlane(h, hull);
        _l4.intersectXYPlane(h, hull);
        _l5.intersectXYPlane(h, hull);
        _l6.intersectXYPlane(h, hull);
        _l7.intersectXYPlane(h, hull);
    }

    private static void printUsage() {
        System.out.println(
            "Usage:\n" +
            "\n" +
            "\tjava Main [-89..89] [-89..89] [-89..89] [-1000..1000]\n" +
            "\n" +
            "Where e.g.\n" +
            "\n" +
            "\tjava Main 54 -34 54 -678\n" +
            "\n" +
            "displays the intersection of the plane 'z = -0.678' and a unit cube\n" +
            "at the origin rotated 54, -34 and 54 degrees about the x, y and\n" +
            "z axes, respectively.");
    }

    private static double parseArg(final String name, final int min, final int max, final double scale, final String input) {
        double result = 0.0;
        try {
            result = Integer.parseInt(input);
            if(result > max || result < min) {
                System.out.println("Error: " + name
                                   + " argument must lie in the range ["
                                   + min + ".." + max
                                   + "]");
                printUsage();
                System.exit(1);
            }
        } catch (NumberFormatException e) {
            System.out.println("Error: " + name + " arguement must be an integer");
            printUsage();
            System.exit(1);
        }
        return result * scale;
    }

    public static void main(final String[] args) {
        if(args.length != 4) {
            printUsage();
            return;
        }

        // alpha, beta, gamma (+/- 89) and h (+/- 1000)
        double alpha = parseArg("First" ,   -89,   89,   1.0, args[0]);
        double beta  = parseArg("Second",   -89,   89,   1.0, args[1]);
        double gamma = parseArg("Third" ,   -89,   89,   1.0, args[2]);
        double h     = parseArg("Fourth", -1000, 1000, 0.001, args[3]);

        JFrame.setDefaultLookAndFeelDecorated(true);
        JFrame frame = new JFrame("Convex Hull");
        frame.setSize(500, 500);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        Cube cube = new Cube();
        cube.rotate(alpha, beta, gamma);

        ConvexHull hull = new ConvexHull();
        cube.intersectXYPlane(h, hull);

        frame.add(new HullPanel(hull));
        frame.setVisible(true);
    }

}
