import java.awt.geom.*;
import javax.swing.*;
import java.awt.*;
import java.util.*;;

@SuppressWarnings("serial")
public class HullPanel extends JPanel {

    private static final double SCALE = 300.0;

    private final java.util.List<Point2D> _points;

    public HullPanel(final ConvexHull hull) {
        _points = hull.getHull();
    }

    private void drawPoint(final double x, final double y, final Graphics2D g) {
        Shape point = new Ellipse2D.Double((SCALE * x) - 3.5, (SCALE * y) - 3.5, 7, 7);
        g.setColor(Color.RED);
        g.fill(point);
    }

    private void drawLine(final Point2D from, final Point2D to, final Graphics2D g) {
        g.drawLine((int)(SCALE * from.getX()),
                   (int)(SCALE * from.getY()),
                   (int)(SCALE * to.getX()),
                   (int)(SCALE * to.getY()));
    }

    @Override
    protected void paintComponent(final Graphics g) {
        super.paintComponent(g);
        Graphics2D gg = (Graphics2D) g;
        gg.translate(250, 250);
        gg.scale(1, -1);

        final int count = _points.size();
        if (count > 1) {

            Path2D outline = new Path2D.Double();
            outline.moveTo(SCALE * _points.get(0).getX(), SCALE * _points.get(0).getY());
            for (int i = 1; i < count; i++) {
                outline.lineTo(SCALE * _points.get(i).getX(), SCALE * _points.get(i).getY());
            }
            outline.closePath();
            gg.setColor(Color.GREEN);
            gg.fill(outline);

            gg.setColor(Color.GREEN.darker());
            gg.setStroke(new BasicStroke(3.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
            for (int i = 0; i < count; i++) {
                drawLine(_points.get(i), _points.get((i+1) % count), gg);
            }
        }

        gg.setColor(Color.RED);
        for (Point2D p : _points) {
            drawPoint(p.getX(), p.getY(), gg);
        }
    }
}
